package authentication.context;

import javax.security.auth.spi.LoginModule;
import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.security.auth.callback.CallbackHandler;

import com.sap.tc.logging.Location;
import com.sap.tc.logging.Severity;

import java.util.Map;

public class ForwardLoginModule implements LoginModule {

  public static final String LOGIN_MODULES_STACK_NAME = "login.modules.stack.name";
  
  private static final Location LOCATION = Location.getLocation("authentication.framework");

  private LoginContext loginContext = null;

  public void initialize(Subject subject, CallbackHandler callbackHandler, Map sharedState, Map options) {
  	if (LOCATION.beDebug()) {
  		LOCATION.debugT("Initializing ForwardLoginModule.");
  	}
  	
    try {
      String loginModuleStack = (String) options.get(LOGIN_MODULES_STACK_NAME);

      loginContext = new CustomLoginContext(loginModuleStack, subject, callbackHandler);
    } catch (Exception e) {
    	loginContext = null;
    	LOCATION.traceThrowableT(Severity.ERROR, "Error during ForwardLoginModule initialization.", e);
    	throw new SecurityException(e.toString());
    }

  	if (LOCATION.beDebug()) {
  		LOCATION.debugT("ForwardLoginModule initialized: " + loginContext);
  	}
  }

  public boolean login() throws LoginException {
  	if (LOCATION.beDebug()) {
  		LOCATION.debugT("ForwardLoginModule login begin.");
  	}
  	
  	if (loginContext != null) {
    	loginContext.login();
  	} else {
    	if (LOCATION.beDebug()) {
    		LOCATION.debugT("ForwardLoginModule login returns false.");
    	}
  		return false;
  	}

  	if (LOCATION.beDebug()) {
  		LOCATION.debugT("ForwardLoginModule login returns true.");
  	}
  	
    return true;
  }

  public boolean commit() throws LoginException {
    return true;
  }

  public boolean abort() throws LoginException {
    return true;
  }

  public boolean logout() throws LoginException {
  	if (LOCATION.beDebug()) {
  		LOCATION.debugT("ForwardLoginModule logout begin.");
  	}
  	
  	if (loginContext != null) {
    	loginContext.logout();
  	} else {
  		if (LOCATION.beDebug()) {
    		LOCATION.debugT("ForwardLoginModule logout returns false.");
    	}
  		
  		return false;
    }
  	
  	if (LOCATION.beDebug()) {
  		LOCATION.debugT("ForwardLoginModule logout returns true.");
  	}
  	
    return true;
  }

}



