package authentication.context;

import java.io.IOException;
 
import javax.security.auth.Subject;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import authentication.callback.HttpCallbackHandler;

import com.sap.engine.services.servlets_jsp.server.application.ApplicationContext;
import com.sap.engine.services.servlets_jsp.server.application.ServletContextImpl;
import com.sap.engine.services.servlets_jsp.server.application.WebApplicationConfig;
import com.sap.tc.logging.Location;
import com.sap.tc.logging.Severity;

public class LoginFilter implements Filter {
	
  private static final Location location = Location.getLocation("authentication.framework");

  String authStack = null;
  WebApplicationConfig applicationConfiguration = null;
 
  public void init(FilterConfig config) throws ServletException {
  	location.infoT("Initializing LoginFilter");
  	ServletContext servletContext = config.getServletContext();
  	ApplicationContext applicationContext = ((ServletContextImpl) servletContext).getApplicationContext();
  	
  	applicationConfiguration = applicationContext.getWebApplicationConfiguration();
  }

  public void destroy() {
  	location.infoT("LoginFilter destroyed.");
  }
 
  public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
  	location.infoT("LoginFilter doFilter");
  	
  	authStack = applicationConfiguration.getAuthType();
  	location.infoT("Authentication stack application configuration: " + authStack);
  	
  	if (authStack == null) {
  		authStack = "other";
  	}
  	
  	location.infoT("Actual authentication stack: " + authStack);
  	
    HttpServletRequest httpRequest = (HttpServletRequest) request;
    HttpServletResponse httpResponse = (HttpServletResponse) response;
    
  	try {
  		HttpCallbackHandler handler = new HttpCallbackHandler(httpRequest, httpResponse, applicationConfiguration);
  		LoginContext loginContext = new CustomLoginContext(authStack, new Subject(), handler);
  		
      loginContext.login();
      location.infoT("LoginFilter login successful");

      if (httpResponse.isCommitted()) {
      	location.infoT("LoginFilter response is commited");
        return;
      }

      chain.doFilter(request, response);
      location.infoT("LoginFilter chain invoked.");
      
    } catch (LoginException le) {
    	location.traceThrowableT(Severity.ERROR, "LoginFilter exception", le);
    }
  }

}

